@@ -1,6 +1,7 @@
-'''
+"""
 A simple dependency injection framework
-'''
+"""
+import dataclasses
 import functools
 import inspect
 from typing import (
@@ -15,6 +16,7 @@ from typing import (
     cast,
     overload,
 )
+from typing_extensions import Protocol
 
 
 class _SentinelClass:
@@ -28,10 +30,10 @@ VT = TypeVar("VT")
 
 
 class Provider(Generic[VT]):
-    '''
+    """
     The base class for Provider implementations. Could be used as the type annotations
     of all the implementations.
-    '''
+    """
 
     STATE_FIELDS: Tuple[str, ...] = ("_override",)
 
@@ -42,25 +44,25 @@ class Provider(Generic[VT]):
         raise NotImplementedError
 
     def set(self, value: Union[_SentinelClass, VT]) -> None:
-        '''
+        """
         set the value to this provider, overriding the original values
-        '''
+        """
         if isinstance(value, _SentinelClass):
             return
         self._override = value
 
     def get(self) -> VT:
-        '''
+        """
         get the value of this provider
-        '''
+        """
         if not isinstance(self._override, _SentinelClass):
             return self._override
         return self._provide()
 
     def reset(self) -> None:
-        '''
+        """
         remove the overriding and restore the original value
-        '''
+        """
         self._override = sentinel
 
     def __getstate__(self) -> Dict[str, Any]:
@@ -72,10 +74,10 @@ class Provider(Generic[VT]):
 
 
 class _ProvideClass:
-    '''
+    """
     Used as the default value of a injected functool/method. Would be replaced by the
     final value of the provider when this function/method gets called.
-    '''
+    """
 
     def __getitem__(self, provider: Provider[VT]) -> VT:
         return provider  # type: ignore
@@ -100,6 +102,9 @@ WrappedCallable = TypeVar("WrappedCallable", bound=Callable[..., Any])
 
 
 def _inject(func: WrappedCallable, squeeze_none: bool) -> WrappedCallable:
+    if getattr(func, "_is_injected", False):
+        return func
+
     sig = inspect.signature(func)
 
     @functools.wraps(func)
@@ -121,6 +126,7 @@ def _inject(func: WrappedCallable, squeeze_none: bool) -> WrappedCallable:
 
         return func(*_inject_args(bind.args), **_inject_kwargs(bind.kwargs))
 
+    setattr(_, "_is_injected", True)
     return cast(WrappedCallable, _)
 
 
@@ -139,29 +145,43 @@ def inject(
 def inject(
     func: Optional[WrappedCallable] = None, squeeze_none: bool = False
 ) -> Union[WrappedCallable, Callable[[WrappedCallable], WrappedCallable]]:
-    '''
+    """
     Used with `Provide`, inject values to provided defaults of the decorated
     function/method when gets called.
-    '''
+    """
     if func is None:
-        wrapped = functools.partial(_inject, squeeze_none=squeeze_none)
-        return cast(Callable[[WrappedCallable], WrappedCallable], wrapped)
+        wrapper = functools.partial(_inject, squeeze_none=squeeze_none)
+        return cast(Callable[[WrappedCallable], WrappedCallable], wrapper)
 
     if callable(func):
         return _inject(func, squeeze_none=squeeze_none)
 
-    raise ValueError('You must pass either int or str')
+    raise ValueError("You must pass either None or Callable")
 
 
-class Container:
-    '''
-    The base class of containers
-    '''
+def sync_container(from_: Any, to_: Any) -> None:
+    for f in dataclasses.fields(to_):
+        src = f.default
+        target = getattr(from_, f.name, None)
+        if target is None:
+            continue
+        if isinstance(src, Provider):
+            src.__setstate__(target.__getstate__())
+        elif dataclasses.is_dataclass(src):
+            sync_container(src, target)
 
-    def __init__(self) -> None:
-        raise TypeError('Container should not be instantiated')
+
+container = dataclasses.dataclass
 
 
 skip = not_passed = sentinel
 
-__all__ = ["Container", "Provider", "Provide", "inject", "not_passed", "skip"]
+__all__ = [
+    "container",
+    "Provider",
+    "Provide",
+    "inject",
+    "not_passed",
+    "skip",
+    "sync_container",
+]
